@@ -4,7 +4,7 @@
 /*           http://open-jtalk.sourceforge.net/                      */
 /* ----------------------------------------------------------------- */
 /*                                                                   */
-/*  Copyright (c) 2008-2012  Nagoya Institute of Technology          */
+/*  Copyright (c) 2008-2013  Nagoya Institute of Technology          */
 /*                           Department of Computer Science          */
 /*                                                                   */
 /* All rights reserved.                                              */
@@ -57,6 +57,17 @@ JPCOMMON_LABEL_C_START;
 
 #include "jpcommon.h"
 
+#ifdef ASCII_HEADER
+#if defined(CHARSET_EUC_JP)
+#include "jpcommon_rule_ascii_for_euc_jp.h"
+#elif defined(CHARSET_SHIFT_JIS)
+#include "jpcommon_rule_ascii_for_shift_jis.h"
+#elif defined(CHARSET_UTF_8)
+#include "jpcommon_rule_ascii_for_utf_8.h"
+#else
+#error CHARSET is not specified
+#endif
+#else
 #if defined(CHARSET_EUC_JP)
 #include "jpcommon_rule_euc_jp.h"
 #elif defined(CHARSET_SHIFT_JIS)
@@ -66,6 +77,7 @@ JPCOMMON_LABEL_C_START;
 #else
 #error CHARSET is not specified
 #endif
+#endif
 
 #define MAXBUFLEN 1024
 #define MAX_S     19
