@@ -4,7 +4,7 @@
 /*           http://open-jtalk.sourceforge.net/                      */
 /* ----------------------------------------------------------------- */
 /*                                                                   */
-/*  Copyright (c) 2008-2012  Nagoya Institute of Technology          */
+/*  Copyright (c) 2008-2013  Nagoya Institute of Technology          */
 /*                           Department of Computer Science          */
 /*                                                                   */
 /* All rights reserved.                                              */
@@ -58,6 +58,17 @@ NJD_SET_DIGIT_C_START;
 #include "njd.h"
 #include "njd_set_digit.h"
 
+#ifdef ASCII_HEADER
+#if defined(CHARSET_EUC_JP)
+#include "njd_set_digit_rule_ascii_for_euc_jp.h"
+#elif defined(CHARSET_SHIFT_JIS)
+#include "njd_set_digit_rule_ascii_for_shift_jis.h"
+#elif defined(CHARSET_UTF_8)
+#include "njd_set_digit_rule_ascii_for_utf_8.h"
+#else
+#error CHARSET is not specified
+#endif
+#else
 #if defined(CHARSET_EUC_JP)
 #include "njd_set_digit_rule_euc_jp.h"
 #elif defined(CHARSET_SHIFT_JIS)
@@ -67,6 +78,7 @@ NJD_SET_DIGIT_C_START;
 #else
 #error CHARSET is not specified
 #endif
+#endif
 
 #define MAXBUFLEN 1024
 
@@ -395,8 +407,10 @@ void njd_set_digit(NJD * njd)
             /* convert digit pron */
             if (search_numerative_class(njd_set_digit_rule_numerative_class1b, node) == 1)
                convert_digit_pron(njd_set_digit_rule_conv_table1b, node->prev);
-            else if (search_numerative_class(njd_set_digit_rule_numerative_class1c, node) == 1)
-               convert_digit_pron(njd_set_digit_rule_conv_table1c, node->prev);
+            else if (search_numerative_class(njd_set_digit_rule_numerative_class1c1, node) == 1)
+               convert_digit_pron(njd_set_digit_rule_conv_table1c1, node->prev);
+            else if (search_numerative_class(njd_set_digit_rule_numerative_class1c2, node) == 1)
+               convert_digit_pron(njd_set_digit_rule_conv_table1c2, node->prev);
             else if (search_numerative_class(njd_set_digit_rule_numerative_class1d, node) == 1)
                convert_digit_pron(njd_set_digit_rule_conv_table1d, node->prev);
             else if (search_numerative_class(njd_set_digit_rule_numerative_class1e, node) == 1)
@@ -468,18 +482,21 @@ void njd_set_digit(NJD * njd)
          }
          if (search_numerative_class(njd_set_digit_rule_numeral_list8, node) == 1)
             convert_digit_pron(njd_set_digit_rule_numeral_list9, node->prev);
+         if (search_numerative_class(njd_set_digit_rule_numeral_list10, node) == 1)
+            convert_digit_pron(njd_set_digit_rule_numeral_list11, node->prev);
          if (search_numerative_class(njd_set_digit_rule_numeral_list6, node) == 1)
             convert_numerative_pron(njd_set_digit_rule_numeral_list7, node->prev, node);
       }
    }
 
-   for (node = njd->head->next; node != NULL; node = node->next) {
+   for (node = njd->head; node != NULL; node = node->next) {
       if (node->next != NULL &&
           strcmp(NJDNode_get_string(node->next), "*") != 0 &&
           strcmp(NJDNode_get_pos_group1(node), NJD_SET_DIGIT_KAZU) == 0 &&
-          strcmp(NJDNode_get_pos_group1(node->prev), NJD_SET_DIGIT_KAZU) != 0 &&
-          (strcmp(NJDNode_get_pos_group2(node->next), NJD_SET_DIGIT_JOSUUSHI) == 0
-           || strcmp(NJDNode_get_pos_group1(node->next), NJD_SET_DIGIT_FUKUSHIKANOU) == 0)) {
+          (node->prev == NULL
+           || strcmp(NJDNode_get_pos_group1(node->prev), NJD_SET_DIGIT_KAZU) != 0)
+          && (strcmp(NJDNode_get_pos_group2(node->next), NJD_SET_DIGIT_JOSUUSHI) == 0
+              || strcmp(NJDNode_get_pos_group1(node->next), NJD_SET_DIGIT_FUKUSHIKANOU) == 0)) {
          /* convert class3 */
          for (i = 0; njd_set_digit_rule_numerative_class3[i] != NULL; i += 2) {
             if (strcmp(NJDNode_get_string(node->next), njd_set_digit_rule_numerative_class3[i]) == 0
@@ -510,7 +527,7 @@ void njd_set_digit(NJD * njd)
          /* the day of month */
          if (strcmp(NJDNode_get_string(node->next), NJD_SET_DIGIT_NICHI) == 0
              && strcmp(NJDNode_get_string(node), "*") != 0) {
-            if (NJDNode_get_string(node->prev) != NULL
+            if (node->prev != NULL
                 && strstr(NJDNode_get_string(node->prev), NJD_SET_DIGIT_GATSU) != NULL
                 && strcmp(NJDNode_get_string(node), NJD_SET_DIGIT_ONE) == 0) {
                NJDNode_load(node, NJD_SET_DIGIT_TSUITACHI);
