@@ -4,7 +4,7 @@
 /*           http://open-jtalk.sourceforge.net/                      */
 /* ----------------------------------------------------------------- */
 /*                                                                   */
-/*  Copyright (c) 2008-2012  Nagoya Institute of Technology          */
+/*  Copyright (c) 2008-2013  Nagoya Institute of Technology          */
 /*                           Department of Computer Science          */
 /*                                                                   */
 /* All rights reserved.                                              */
@@ -176,12 +176,23 @@ static const char *njd_set_digit_rule_numeral_list7[] = {
 };
 
 static const char *njd_set_digit_rule_numeral_list8[] = {
-   "ɴ", "", NULL
+   "ɴ", NULL
 };
 
 static const char *njd_set_digit_rule_numeral_list9[] = {
    "ϻ", "", "0", "2",
-   "Ȭ", "ϥ", "1", "2",
+   "Ȭ", "ϥ", "0", "2",
+   NULL, NULL, NULL
+};
+
+static const char *njd_set_digit_rule_numeral_list10[] = {
+   "", "", NULL
+};
+
+static const char *njd_set_digit_rule_numeral_list11[] = {
+   "", "", "0", "2",
+   "Ȭ", "ϥ", "0", "2",
+   "", "", "1", "2",
    NULL, NULL, NULL
 };
 
@@ -198,7 +209,7 @@ static const char *njd_set_digit_rule_conv_table1b[] = {
    NULL, NULL, NULL, NULL
 };
 
-static const char *njd_set_digit_rule_numerative_class1c[] = {
+static const char *njd_set_digit_rule_numerative_class1c1[] = {
    /* from paper */
    "", "", "",
    /* from dictionary */
@@ -206,9 +217,21 @@ static const char *njd_set_digit_rule_numerative_class1c[] = {
    NULL
 };
 
-static const char *njd_set_digit_rule_conv_table1c[] = {
+static const char *njd_set_digit_rule_conv_table1c1[] = {
    "", "", "0", "1",
    "", "", "1", "2",
+   NULL, NULL, NULL, NULL
+};
+
+static const char *njd_set_digit_rule_numerative_class1c2[] = {
+   /* from paper */
+   "", "",
+   /* from dictionary */
+   "", "Ⱦ",
+   NULL
+};
+
+static const char *njd_set_digit_rule_conv_table1c2[] = {
    "", "", "0", "1",
    NULL, NULL, NULL, NULL
 };
@@ -295,7 +318,7 @@ static const char *njd_set_digit_rule_numerative_class1h[] = {
    "", "", "", "", "", "·", "", "Ĥ", "Ĥ", "Ĥ", "Ĥޤ", "",
    "ޤ", "Ȥ", "Ȥ", "Ȥ", "Ȥ", "", "", "", "«", "³", "", "",
    "", "î", "ȿ", "", "", "Ļ", "", "Ϥ", "", "ũ", "Ź", "ž", "", "", "",
-   "", "", "", "", "", "ʬ", "", "", "", "γ",
+   "", "", "", "", "", "ʬ", "", "", "", "γ", "", "", "",
    NULL
 };
 
