@@ -4,7 +4,7 @@
 /*           http://open-jtalk.sourceforge.net/                      */
 /* ----------------------------------------------------------------- */
 /*                                                                   */
-/*  Copyright (c) 2008-2012  Nagoya Institute of Technology          */
+/*  Copyright (c) 2008-2013  Nagoya Institute of Technology          */
 /*                           Department of Computer Science          */
 /*                                                                   */
 /* All rights reserved.                                              */
@@ -176,12 +176,23 @@ static const char *njd_set_digit_rule_numeral_list7[] = {
 };
 
 static const char *njd_set_digit_rule_numeral_list8[] = {
-   "S", "", NULL
+   "S", NULL
 };
 
 static const char *njd_set_digit_rule_numeral_list9[] = {
    "Z", "b", "0", "2",
-   "", "nb", "1", "2",
+   "", "nb", "0", "2",
+   NULL, NULL, NULL
+};
+
+static const char *njd_set_digit_rule_numeral_list10[] = {
+   "", "", NULL
+};
+
+static const char *njd_set_digit_rule_numeral_list11[] = {
+   "", "Cb", "0", "2",
+   "", "nb", "0", "2",
+   "\", "Wb", "1", "2",
    NULL, NULL, NULL
 };
 
@@ -198,7 +209,7 @@ static const char *njd_set_digit_rule_conv_table1b[] = {
    NULL, NULL, NULL, NULL
 };
 
-static const char *njd_set_digit_rule_numerative_class1c[] = {
+static const char *njd_set_digit_rule_numerative_class1c1[] = {
    /* from paper */
    "", "", "l",
    /* from dictionary */
@@ -206,9 +217,21 @@ static const char *njd_set_digit_rule_numerative_class1c[] = {
    NULL
 };
 
-static const char *njd_set_digit_rule_conv_table1c[] = {
+static const char *njd_set_digit_rule_conv_table1c1[] = {
    "l", "", "0", "1",
    "", "V`", "1", "2",
+   NULL, NULL, NULL, NULL
+};
+
+static const char *njd_set_digit_rule_numerative_class1c2[] = {
+   /* from paper */
+   "", "",
+   /* from dictionary */
+   "", "",
+   NULL
+};
+
+static const char *njd_set_digit_rule_conv_table1c2[] = {
    "", "N", "0", "1",
    NULL, NULL, NULL, NULL
 };
@@ -295,7 +318,7 @@ static const char *njd_set_digit_rule_numerative_class1h[] = {
    "", "I", "K", "w", "", "", "", "", "", "", "܂", "",
    "܂", "Ƃ", "Ƃ", "Ƃ", "Ƃ", "", "", "", "", "", "", "",
    "", "M", "", "", "", "", "", "͂", "", "H", "X", "]", "_", "l", "",
-   "", "", "", "", "n", "", "", "", "", "",
+   "", "", "", "", "n", "", "", "", "", "", "", "Δn", "I",
    NULL
 };
 
