@@ -4,7 +4,7 @@
 /*           http://open-jtalk.sourceforge.net/                      */
 /* ----------------------------------------------------------------- */
 /*                                                                   */
-/*  Copyright (c) 2008-2012  Nagoya Institute of Technology          */
+/*  Copyright (c) 2008-2013  Nagoya Institute of Technology          */
 /*                           Department of Computer Science          */
 /*                                                                   */
 /* All rights reserved.                                              */
@@ -58,6 +58,17 @@ NJD_SET_UNVOICED_VOWEL_C_START;
 #include "njd.h"
 #include "njd_set_unvoiced_vowel.h"
 
+#ifdef ASCII_HEADER
+#if defined(CHARSET_EUC_JP)
+#include "njd_set_unvoiced_vowel_rule_ascii_for_euc_jp.h"
+#elif defined(CHARSET_SHIFT_JIS)
+#include "njd_set_unvoiced_vowel_rule_ascii_for_shift_jis.h"
+#elif defined(CHARSET_UTF_8)
+#include "njd_set_unvoiced_vowel_rule_ascii_for_utf_8.h"
+#else
+#error CHARSET is not specified
+#endif
+#else
 #if defined(CHARSET_EUC_JP)
 #include "njd_set_unvoiced_vowel_rule_euc_jp.h"
 #elif defined(CHARSET_SHIFT_JIS)
@@ -67,6 +78,7 @@ NJD_SET_UNVOICED_VOWEL_C_START;
 #else
 #error CHARSET is not specified
 #endif
+#endif
 
 #define MAXBUFLEN 1024
 
