@@ -4,7 +4,7 @@
 /*           http://open-jtalk.sourceforge.net/                      */
 /* ----------------------------------------------------------------- */
 /*                                                                   */
-/*  Copyright (c) 2008-2012  Nagoya Institute of Technology          */
+/*  Copyright (c) 2008-2013  Nagoya Institute of Technology          */
 /*                           Department of Computer Science          */
 /*                                                                   */
 /* All rights reserved.                                              */
@@ -56,6 +56,17 @@ TEXT2MECAB_C_START;
 
 #include "text2mecab.h"
 
+#ifdef ASCII_HEADER
+#if defined(CHARSET_EUC_JP)
+#include "text2mecab_rule_ascii_for_euc_jp.h"
+#elif defined(CHARSET_SHIFT_JIS)
+#include "text2mecab_rule_ascii_for_shift_jis.h"
+#elif defined(CHARSET_UTF_8)
+#include "text2mecab_rule_ascii_for_utf_8.h"
+#else
+#error CHARSET is not specified
+#endif
+#else
 #if defined(CHARSET_EUC_JP)
 #include "text2mecab_rule_euc_jp.h"
 #elif defined(CHARSET_SHIFT_JIS)
@@ -65,6 +76,7 @@ TEXT2MECAB_C_START;
 #else
 #error CHARSET is not specified
 #endif
+#endif
 
 static int strtopcmp(const char *str, const char *pattern)
 {
